#include <GLUT/glut.h>									//GLUT library
#include "UniformGrid.h"
#include "GlyphRenderer.h"
#include <iostream>										
#include <math.h>										//For expf()



float Xmin	= -5, Xmax	= 5;							//Range of variable x that we sample
float Ymin	= -5, Ymax	= 5;							//Range of variable y that we sample
int   Nx	= 20, Ny	= 20;							//Number of samples taken along the x and y axes



float f(float x, float y)								//A simple two-variable function to plot 
{														//The function is samples 
	float ret = 8*expf(-(x*x+y*y)/5);
	return ret;
}


UniformGrid			grid(Nx,Ny,Xmin,Ymin,Xmax,Ymax);
GlyphRenderer		renderer;
int					drawing_style = 0;
int					colormap      = 0;
int					scaling_mode  = 0;
float				alpha         = 0;


void viewing(int W, int H)								//Window resize function, sets up viewing parameters (GLUT callback function)
{
	glMatrixMode(GL_MODELVIEW);							//1. Set the modelview matrix (including the camera position and view direction)
	glLoadIdentity ();										

	glMatrixMode (GL_PROJECTION);						//2. Set the projection matrix
	glLoadIdentity ();
	gluOrtho2D(Xmin,Xmax,Ymin,Ymax);
	glDisable(GL_DEPTH_TEST);

	glViewport(0,0,W,H);								//3. Set the viewport to the entire size of the rendering window
}



void draw()												//Render the grid (GLUT callback function)
{
	renderer.draw(grid);
}



void createGrid(float alpha)							//Create vectors for the grid. For this, we simply sample the 
{														//rotated gradient of the function f(x,y) at all the grid points
	for(int i=0;i<grid.numPoints();++i)
	{
	   float p[2];				
	   grid.getPoint(i,p);								//Get coordinates of current grid point
	   
	   float scalar = f(p[0],p[1]);						//Evaluate f(x,y) at current point
		
	   float delta    = 0.001*(Xmax-Xmin);				//Evaluate gradient of f(x,y) using finite differences
	   float scalar_x = f(p[0]+delta,p[1]);	
	   float scalar_y = f(p[0],p[1]+delta);		   
	   Point3d gradient = Point3d((scalar_x-scalar)/delta,(scalar_y-scalar)/delta,0);
	   
	   Point3d vec;										//Rotate the gradient with an angle alpha, to generate more complex vector fields
	   vec.x = cos(alpha)*gradient.x - sin(alpha)*gradient.y;
	   vec.y = sin(alpha)*gradient.x + cos(alpha)*gradient.y;
	   	   
	   grid.pointVectors().setC0Vector(i,vec);			//Finally, store the rotated gradient as vector values in the grid
	}   
}



void keyboard(unsigned char c,int,int)					//Keyboard callback
{
	switch(c)
	{
	case ' ':
		{
		  drawing_style = (++drawing_style) % 4;		//Ask renderer to use another drawing-style
		  renderer.setDrawingStyle((GlyphRenderer::DRAW_STYLE)drawing_style);
		  break;
		}
	case 'c':
		{
		  colormap = (++colormap) % 3;					//Ask renderer to use another colormap
		  renderer.setColormapType((GlyphRenderer::COLORMAP_TYPE)colormap);
		  break;
		}
	case '+':											//Increase glyph size
	case '=':
		{
		  renderer.setScalingFactor(1.1*renderer.getScalingFactor());
		  break;
		}  
	case '-':											//Decrease glyph size
	case '_':
		{
		  renderer.setScalingFactor(renderer.getScalingFactor()/1.1);
		  break;
		}  
	case 's':											//Change vector-to-glyph scaling mode
		{
		  scaling_mode = (++scaling_mode) % 2;	
		  renderer.setScalingMode((GlyphRenderer::SCALING_MODE)scaling_mode);
		  break;
		}  
	case 'v':											//Change the vector field to visualize
		{
		  alpha = (alpha + M_PI/10);
		  createGrid(alpha);
		}
	}
				
	glutPostRedisplay();    
}	



int main(int argc,char* argv[])								//Main program
{
   cout<<"In the graphics window, press"<<endl;
   cout<<"  space: toggle through various glyph types"<<endl;
   cout<<"  c:     toggle through various color maps"<<endl;
   cout<<"  s:     scale glyphs with vector magnitude on/off"<<endl;
   cout<<"  +,-:   increase/decrease glyph size"<<endl;

   glutInit(&argc, argv);									//1. Initialize the GLUT toolkit
   glutInitDisplayMode(GLUT_RGB| GLUT_DOUBLE|GLUT_DEPTH);	//2. Ask GLUT to create next windows with a RGB framebuffer and a Z-buffer too
   glutInitWindowSize(500,500);								//3. Tell GLUT how large are the windows we want to create next
   glutCreateWindow("1. Uniform grid (vector glyphs)");		
															//4. Create our window
   
   createGrid(alpha);										//5. Create vector values for the grid vertices
   
   glutKeyboardFunc(keyboard);   
   glutDisplayFunc(draw);									//6. Add a drawing callback to the window	
   glutReshapeFunc(viewing);								//7. Add a resize callback to the window
   glutMainLoop();											//8. Start the event loop that displays the graph and handles window-resize events
   
   return 0;
}




